%global __cmake_builddir %_target_platform

# note: when updating this package to sync with llvm version, please
# sync the mlir package before.

# Please keep this package sync with FC
%global baserelease 1

# Reduce then number of parallel building processes to avoid oom-killer
#global _smp_ncpus_max 4

%global maj_ver 15
%global min_ver 0
%global patch_ver 6
#global rc_ver 3
%global flang_version %{maj_ver}.%{min_ver}.%{patch_ver}
%global flang_srcdir flang-%{flang_version}%{?rc_ver:rc%{rc_ver}}.src

%define major		%{maj_ver}.%{min_ver}
%define minor		1

%define libname	%mklibname flang %{major}
%define devname	%mklibname flang -d

Name:		flang
Version:	%{flang_version}%{?rc_ver:~rc%{rc_ver}}
Release:	%mkrel %{?rc_ver:0.}%{baserelease}%{?rc_ver:.rc%{rc_ver}}
Summary:	A Fortran language front-end designed for integration with LLVM
Group:		Development/Other
License:	ASL 2.0 with exceptions
URL:		https://github.com/llvm/llvm-project/tree/master/flang
Source0:	https://github.com/llvm/llvm-project/releases/download/llvmorg-%{flang_version}%{?rc_ver:-rc%{rc_ver}}/%{flang_srcdir}.tar.xz
Source1:	https://github.com/llvm/llvm-project/releases/download/llvmorg-%{flang_version}%{?rc_ver:-rc%{rc_ver}}/%{flang_srcdir}.tar.xz.sig
Source2:	https://releases.llvm.org/release-keys.asc

# flang depends on one internal clang tablegen file for documentation generation.
Source3:	https://raw.githubusercontent.com/llvm/llvm-project/llvmorg-%{flang_version}%{?rc_ver:-rc%{rc_ver}}/clang/include/clang/Driver/Options.td

# Needed for documentation generation
Patch1: 0001-PATCH-flang-Disable-use-of-sphinx_markdown_tables.patch
Patch2: link-against-libclang-cpp.patch

# TODO: Can be dropped for LLVM 16. The first one is a plain backport, and the second
# one has been upstreamed as https://reviews.llvm.org/D131475.
Patch3: 0001-flang-docs-nfc-Refine-FlangOptionsDocs.td.patch
Patch4: 0001-Use-find_program-for-clang-tblgen.patch

# Exclude armv7 because there isn't enough virtual memory to build on armv7 ("caught: virtual memory exhausted: Cannot allocate memory").
# Exclude i586 building too for now (error on runtime/command.cpp:33:59: error: narrowing conversion...).
ExcludeArch: %{arm} %{ix86}

# Avoid gcc reaching 4GB of memory on 32-bit targets and also running out of
# memory on builders with many CPUs.
%ifarch %{ix86} s390x ppc64le aarch64 %{arm}
%global _lto_cflags %{nil}
%global _smp_mflags -j1
%global _smp_ncpus_max 4
%endif

# We don't produce debug info on ARM to avoid OOM during the build.
%ifarch %{arm}
%global debug_package %{nil}
%endif

BuildRequires:	gcc
BuildRequires:	gcc-c++
BuildRequires:	cmake
BuildRequires:	zlib-devel
BuildRequires:	llvm-devel = %{version}
BuildRequires:	llvm-test = %{version}
BuildRequires:	llvm-googletest = %{version}
BuildRequires:	mlir-devel = %{version}
BuildRequires:	ninja-build
BuildRequires:	python3-lit >= 12.0.0
BuildRequires:	python3-sphinx
BuildRequires:	python3-recommonmark

# The new flang drive requires clang-devel
BuildRequires:	clang-devel = %{version}

# For origin certification
BuildRequires:	gnupg2

# flang is not yet able to compile a running binary, thus it requires
# an external fortran compiler, set in F18_FC env var, which by default
# is set to "gfortran".
Requires:	gcc-gfortran

%description
Flang is a ground-up implementation of a Fortran front end written in modern
C++.

%package -n %libname
Summary:	Runtime libraries for flang
Group:		System/Libraries

%description -n %libname
This package contains the runtime libraries for flang.


%package -n %devname
Summary:	Development header files for flang
Group:		Development/Other
Provides:	flang-devel = %version-%release
Requires:	%{libname} = %{version}-%{release}

%description -n %devname
Flang header files.

%package doc
Summary:	Documentation for Flang
Group:		Documentation
Requires:	%{name} >= %{version}-%{release}

%description doc
Documentation for Flang.

%prep
%{gpgverify} --keyring='%{SOURCE2}' --signature='%{SOURCE1}' --data='%{SOURCE0}'
%autosetup -n %{flang_srcdir} -p2
# Copy Options.td for docs generation
mkdir -p ../clang/include/clang/Driver
cp %{SOURCE3} ../clang/include/clang/Driver

%build
%cmake -GNinja \
	-DMLIR_TABLEGEN_EXE=%{_bindir}/mlir-tblgen \
	-DCMAKE_BUILD_TYPE=RelWithDebInfo \
	-DCMAKE_INSTALL_RPATH=";" \
	-DCLANG_DIR=%{_libdir}/cmake/clang \
	-DLLVM_MAIN_SRC_DIR=%{_datadir}/llvm/src \
	-DBUILD_SHARED_LIBS:BOOL=ON \
	-DLLVM_LINK_LLVM_DYLIB:BOOL=ON \
	-DLLVM_EXTERNAL_LIT=%{_bindir}/lit \
	-DCMAKE_PREFIX_PATH=%{_libdir}/cmake/llvm/ \
\
	-DFLANG_INCLUDE_DOCS:BOOL=ON \
	-DLLVM_ENABLE_SPHINX:BOOL=ON \
	-DSPHINX_WARNINGS_AS_ERRORS=OFF \
	-DSPHINX_EXECUTABLE=%{_bindir}/sphinx-build-3 \
\
%if 0%{?__isa_bits} == 64
	-DLLVM_LIBDIR_SUFFIX=64
%else
	-DLLVM_LIBDIR_SUFFIX=
%endif

# Avoid gcc reaching 4GB of memory
%ifarch %{ix86} s390x x86_64
sed -i -e 's/-g /-g1 /g' -e 's/-O2/-O1/g' */build.ninja
%endif
# On ARM, disable debuginfo entirely to avoid OOM.
%ifarch %{arm}
sed -i -e 's/-g /-g0 /g' -e 's/-O2/-O1/g' */build.ninja
%endif

export LD_LIBRARY_PATH=%{_builddir}/%{flang_srcdir}/%{_vpath_builddir}/lib
%cmake_build
%cmake_build --target docs-flang-html

%install
%cmake_install
# this is a test binary
rm -f %{buildroot}%{_bindir}/f18-parse-demo

install -d %{buildroot}%{_pkgdocdir}/html
cp -r %{_vpath_builddir}/docs/html/* %{buildroot}%{_pkgdocdir}/html/

%check

# Assertion failure: lib/Semantics/canonicalize-acc.cpp:93
# /usr/include/c++/11/optional:447: constexpr const _Tp& std::_Optional_base_impl<_Tp, _Dp>::_M_get() const [with _Tp = Fortran::parser::DoConstruct; _Dp = std::_Optional_base<Fortran::parser::DoConstruct, false, false>]: Assertion 'this->_M_is_engaged()' failed.
rm test/Semantics/OpenACC/acc-canonicalization-validity.f90 || :

# these tests fail on all or specific arches
rm test/Semantics/resolve63.f90 || :
# These tests fail on 32-bit targets.
%ifarch %{ix86} %{arm}
rm -f test/Fir/fir-ops.fir
rm -f test/Semantics/assign03.f90
rm -f test/Semantics/data05.f90
rm -f test/Semantics/offsets01.f90
rm -f test/Semantics/offsets02.f90
rm -f test/Semantics/typeinfo01.f90
rm -f test/Semantics/spec-expr.f90
rm -f test/Evaluate/folding19.f90
%endif

%cmake_build --target check-flang 

%files
%license LICENSE.TXT
%{_bindir}/tco
%{_bindir}/bbc
%{_bindir}/fir-opt
%{_bindir}/flang-new
%{_bindir}/flang-to-external-fc

%files -n %libname
%{_libdir}/libFortranLower.so.%{maj_ver}{,.*}
%{_libdir}/libFortranSemantics.so.%{maj_ver}{,.*}
%{_libdir}/libFortranCommon.so.%{maj_ver}{,.*}
%{_libdir}/libFortranRuntime.so.%{maj_ver}{,.*}
%{_libdir}/libFortranDecimal.so.%{maj_ver}{,.*}
%{_libdir}/libFortranEvaluate.so.%{maj_ver}{,.*}
%{_libdir}/libFortranParser.so.%{maj_ver}{,.*}
%{_libdir}/libflangFrontend.so.%{maj_ver}*
%{_libdir}/libflangFrontendTool.so.%{maj_ver}*
%{_libdir}/libFIRBuilder.so.%{maj_ver}
%{_libdir}/libFIRCodeGen.so.%{maj_ver}
%{_libdir}/libFIRDialect.so.%{maj_ver}
%{_libdir}/libFIRSupport.so.%{maj_ver}
%{_libdir}/libFIRTransforms.so.%{maj_ver}


%files -n %devname
%{_libdir}/libFortranLower.so
%{_libdir}/libFortranParser.so
%{_libdir}/libFortranCommon.so
%{_libdir}/libFortranSemantics.so
%{_libdir}/libFortran_main.a
%{_libdir}/libFIRBuilder.so
%{_libdir}/libFIRCodeGen.so
%{_libdir}/libFIRDialect.so
%{_libdir}/libFIRSupport.so
%{_libdir}/libFIRTransforms.so
%{_libdir}/libFortranDecimal.so
%{_libdir}/libFortranRuntime.so
%{_libdir}/libFortranEvaluate.so
%{_libdir}/libflangFrontend.so
%{_libdir}/libflangFrontendTool.so
%{_includedir}/flang
%{_libdir}/cmake/

%files doc
%dir %{_pkgdocdir}
%doc %{_pkgdocdir}/html/


%changelog
* Thu Dec 08 2022 ghibo <ghibo> 15.0.6-1.mga9
+ Revision: 1919343
- Add a note to spec file
+ tv <tv>
- Update to 15.0.6
- Update to 15.0.6

* Fri Nov 04 2022 ghibo <ghibo> 15.0.4-3.mga9
+ Revision: 1902641
- Update to release 15.0.4

* Thu Nov 03 2022 ghibo <ghibo> 15.0.0-3.mga9
+ Revision: 1902021
- Don't build doc as noarch, as package is not available on any arch

* Sun Oct 23 2022 ghibo <ghibo> 15.0.0-2.mga9
+ Revision: 1898768
- Re-introduced aarch64

* Tue Sep 27 2022 ghibo <ghibo> 15.0.0-1.mga9
+ Revision: 1892869
- Fix filelist
- Add Source3 (merged from FC) to build docs.
- Add Patch3, Patch4 (merged from FC) to build docs.
- Drop Patch3
- Update Patch2
- Update release-keys.asc
- Downgrade to release 15.0.0
- Update to release 15.0.1

* Tue Jun 21 2022 tv <tv> 14.0.5-1.mga9
+ Revision: 1865345
- Update to 14.0.5
+ ghibo <ghibo>
- Update to release 14.0.3

* Thu May 19 2022 ghibo <ghibo> 14.0.0-1.mga9
+ Revision: 1859306
- Exclude i586 build due to error on number conversions
- Don't reduce ncpus_max to 4 when building under x86_64 arch.
- Enable parallel building for x86_64.
- Remove setting LD_LIBRARY_PATH before invoking cmake_build check-flang, to let check pass.
- Fix tabs/spaces (rpmlint).
- Let the %%check stage to be compliant with short-circuit building.
+ tv <tv>
- Fix building with gcc-12
- Update to LLVM 14.0.0

* Fri Feb 11 2022 tv <tv> 13.0.1-1.mga9
+ Revision: 1773026
- Fixup sed calls (cmake macro changed vs paths??)
- Disable debuginfo on arm (in order to avoir OOM)
- Update to 13.0.1 final
- 13.0.0 Release

* Mon Jul 19 2021 tv <tv> 12.0.1-1.mga9
+ Revision: 1737071
- 12.0.1

* Fri Apr 16 2021 tv <tv> 12.0.0-1.mga9
+ Revision: 1715998
- 12.0.0 final

* Fri Apr 09 2021 tv <tv> 12.0.0-0.6.rc5.mga9
+ Revision: 1714303
- 12.0.0 RC5

* Fri Apr 02 2021 tv <tv> 12.0.0-0.4.rc4.mga9
+ Revision: 1712642
- 12.0.0 RC4

* Fri Mar 12 2021 tv <tv> 12.0.0-0.4.rc3.mga9
+ Revision: 1701744
- LLVM 12.0.0 rc3

* Thu Mar 04 2021 tv <tv> 12.0.0-0.2.rc2.mga9
+ Revision: 1698006
- 12.0.0-rc2 Release

* Thu Jan 07 2021 ghibo <ghibo> 11.0.1-3.mga8
+ Revision: 1669684
- Reduce _smp_ncpus_max to avoid oom-killer
+ tv <tv>
- LLVM 11.0.1 final

* Thu Dec 31 2020 ghibo <ghibo> 11.0.1-2.rc2.mga8
+ Revision: 1667288
- Update to release 11.0.1rc2

* Fri Dec 04 2020 ghibo <ghibo> 11.0.1-2.rc1.mga8
+ Revision: 1652442
- Add gfortran in Requires
- Reduce amount of parallel building for aarch64 and arm
- Exclude building on arm and aarch64
- imported package flang


* Wed Dec 02 2020 ghibo <ghibo> 11.0.1-rc1-1.mga8
- initial release, synched from FC.
k